@@ -1,4 +1,4 @@
-// $Id: main_thread.c,v 1.9 2006/03/15 19:23:04 khlut Exp $
+// $Id: main_thread.c,v 1.10 2006/03/19 20:17:14 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -20,6 +20,7 @@
 #include "genres.h"
 #include <sys/wait.h>
 #include <signal.h>
+#include <time.h>
 
 #define LICENSE "Copyright 2005 Elphel, Inc.<br>"\
 "This plug-in is free software; you can redistribute it and/or modify"\
@@ -73,7 +74,7 @@ char * objectprop(NPP instance, int object_type, const char *name)
     NPObject *a=0; char  *s0, *s1, *n;
     NPIdentifier id;
     NPVariant var={NPVariantType_Void};
-    Debug(" objectprop\n");
+    Debug("Objectprop");
     if(NPN.getvalue(instance, object_type, &a))	return 0;
     Debug(" %d\n",a->referenceCount);
     s0=n=strdup(name);
@@ -101,11 +102,14 @@ char * objectprop(NPP instance, int object_type, const char *name)
 /*
 ** Variable manipulation
 */
-
+int do_send_value(PD*pd, char *key, char *value) {
+  Debug("%p send %s=%s\n",pd,key,value);
+  return fprintf(pd->slavectrl,"%s=%s\n",key,value);
+}
 void send_value(PD *pd, char *key, char *value) //send variable to slave
 {
   PR_Lock(pd->lock);
-  if(( (!pd->slavectrl) || fprintf(pd->slavectrl,"%s=%s\n",key,value)<=0 ) && pd->status==Running // try to reanimate slave
+  if(( (!pd->slavectrl) || do_send_value(pd,key,value)<=0 ) && pd->status==Running // try to reanimate slave
      && ( strcmp(key,"run") || strcmp(value,"0") )) { 	// ignore run=0 when slave is nonfunctional
     if(debug)perror("write to pipe");
    
@@ -123,12 +127,11 @@ void send_value(PD *pd, char *key, char *value) //send variable to slave
     for(i=0;i<pd->vars->size;i++) {
 	if(pd->vars->var[i].key==run || pd->vars->var[i].key==ready) continue;
 	char *sv=NPVariant_to_string(&(pd->vars->var[i].value));
-	fprintf(pd->slavectrl,"%s=%s\n",NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
-	Debug("send %s=%s\n",NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
+	do_send_value(pd, NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
 	free(sv);
     }
     pd->xid && fprintf(pd->slavectrl,"xid=%x\n",(unsigned)pd->xid);
-    fprintf(pd->slavectrl,"%s=%s\n",key,value);
+    do_send_value(pd,key,value);
     pd->status=Running;
     
     return;
@@ -188,7 +191,7 @@ typedef struct {
 NPObject * NPAllocate(NPP npp, NPClass *aClass)
 {
     PO *po=PR_Malloc(sizeof(PO));
-    Debug("OBJ-ALLOCATE\n");
+    Debug("Obj allocate %p\n", npp->pdata);
     if(po) {
 	po->npo._class=&genresClass;
 	po->npo.referenceCount=0;
@@ -198,7 +201,7 @@ NPObject * NPAllocate(NPP npp, NPClass *aClass)
 }
 void NPDeallocate(NPObject *npobj)
 {
-    Debug("OBJ-DEALLOCATE %d\n", npobj? npobj->referenceCount : -100);
+    Debug("Obj deallocate %p\n", ((PO*)npobj)->instance->pdata);
     PR_Free(npobj);
 }
 bool NPHasMethod(NPObject *npobj, NPIdentifier name)
@@ -265,11 +268,11 @@ NPError NP_GetValue(void *future, NPPVariable variable, void *value)
   Debug("NP_GETVALUE\n");
   switch (variable) {
     case NPPVpluginNameString:
-      *((char **) value) = "The GenReS Mozilla plugin V 0.8.3";
+      *((char **) value) = "The GenReS Mozilla plugin V " RELEASE;
       break;
     case NPPVpluginDescriptionString:
       *((char **) value) = "<a href=https://savannah.nongnu.org/projects/genres>GENeric REconfigureable Scriptable plugin</a> for Mozilla/Firefox<br>"
-    			   "Version 0.8.3<br>"
+    			   "Version " RELEASE "<br>"
 			   "Author: Sergey Khlutchin<br>"
 			   LICENSE;
       break;
@@ -371,11 +374,11 @@ NPError NPP_New(NPMIMEType mime, NPP instance, uint16 mode, int16 argc,
     for(i=0; i<argc; i++) send_value(pd,argn[i],argv[i]);
     pd->status=Running;
     
-    Debug("pdata refcnt=%d\n", Pd->refcnt);
+    Debug("%p refcnt=%d\n",pd, Pd->refcnt);
     process_pending_events(Pd);
-    Debug("pdata refcnt=%d\n", Pd->refcnt);
+    Debug("%p refcnt=%d\n",pd, Pd->refcnt);
     process_pending_events(Pd);
-    Debug("pdata refcnt=%d\n", Pd->refcnt);
+    Debug("%p refcnt=%d\n",pd, Pd->refcnt);
     
     return NPERR_NO_ERROR;
 }
@@ -395,7 +398,7 @@ void wait_slave(PD * pd) { // wait for slave termination
     NPVariant *var;	int i;
     do{
         PR_Sleep(100);
-        Debug("Process events %x\n", (int)pd->instance);
+        Debug("%p Process events\n", pd);
         process_pending_events(pd);
         var=storage_get(pd->vars, ready);
         if(!var || var->type!=NPVariantType_String) {
@@ -404,7 +407,7 @@ void wait_slave(PD * pd) { // wait for slave termination
         }
     }while(var->value.stringValue.utf8characters[0]=='1');
 
-    Debug("pdata refcnt=%d\n", pd->refcnt);
+    Debug("%p refcnt=%d\n", pd, pd->refcnt);
     
     pd->status=Cancelling;
     PR_Sleep(100);
@@ -414,16 +417,16 @@ void wait_slave(PD * pd) { // wait for slave termination
     Debug("wait observer\n");
     if(pd->observer) {PR_JoinThread(pd->observer); pd->observer=0;}
     if(pd->slavectrl) {fclose(pd->slavectrl); pd->slavectrl=0;}
-    Debug("Process events %x\n", (int)pd->instance);
+    Debug("Process events %p\n", pd);
     process_pending_events(pd);
-    Debug("pdata refcnt=%d\n", pd->refcnt);
+    Debug("%p refcnt=%d\n", pd, pd->refcnt);
     process_pending_events(pd);
-    Debug("pdata refcnt=%d\n", pd->refcnt);
+    Debug("%p refcnt=%d\n", pd, pd->refcnt);
 }
 
 NPError NPP_Destroy (NPP instance, NPSavedData** save)
 {
-    Debug("NP_Destroy\n");
+    Debug("%p NPP_Destroy\n", instance->pdata);
     if(Pd) {
       if(Pd->observer && Pd->status==Running) {
 	send_value_hex(Pd,"run", 0);
@@ -440,7 +443,7 @@ NPError NPP_Destroy (NPP instance, NPSavedData** save)
 
 NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value)
 {
-    Debug("NPP_GETVALUE %d %x\n", (int) variable, (int)value);
+    Debug("%p NPP_GETVALUE %d %x\n", instance->pdata, (int) variable, (int)value);
     switch(variable) {
     case NPPVpluginScriptableNPObject: {
 	*((NPObject **) value) = NPN.createobject(instance, &genresClass);
@@ -461,7 +464,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 {
     Window xid= (Window)npwindow->window;
     
-    Debug("NPP_SETWINDOW %x %x %x %x\n", (int) instance, (int)xid,
+    Debug("%p NPP_SETWINDOW %x %x %x\n", instance->pdata, (int)xid,
            (int)npwindow->ws_info, (int)npwindow->type );
     
     if(Pd->xid!=xid) {
@@ -475,7 +478,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 
 NPError NPP_NewStream(NPP instance, NPMIMEType type, NPStream* stream, NPBool seekable, uint16* stype)
 {
-    Debug("NPP_NEWSTREAM %x %x\n", (int) instance, (int)seekable);
+    Debug("%p NPP_NEWSTREAM %x\n", instance->pdata, (int)seekable);
     return NPERR_GENERIC_ERROR;
 }
 
@@ -502,7 +505,7 @@ PLEvent * strevent( PD *pd, const char * str)  // make event from a string (is c
     if (!event)		{ PR_Unlock(pd->lock); return 0; }
 
     if(strncmp(str,"debug=",6)==0) debug=str[6]-'0';
-    Debug("newEvent %u %s\n", (int)event, str);
+    Debug("%p %d newEvent %s\n", pd, time(0), str);
     
     PL_InitEvent( &event->e, pd, HandleEvent, DestroyEvent);
     event->str=PR_Malloc(strlen(str)+3);
@@ -518,8 +521,7 @@ void PR_CALLBACK DestroyEvent(ObserveEvent* event)
 {
     if(event) {
 	PR_Lock(event->pd->lock);
-	Debug("DestroyEvent %u ", (int)event);
-	Debug("pdata refcnt=%d\n", event->pd->refcnt);
+	Debug("%p DestroyEvent refcnt=%d\n", event->pd, event->pd->refcnt);
 	free_pd(event->pd);
 	if(event->str) PR_Free(event->str);
 	PR_Free(event);
@@ -528,7 +530,7 @@ void PR_CALLBACK DestroyEvent(ObserveEvent* event)
 
 void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 {
-    Debug("HandleEvent %d \"%s\"\t", (int)event, event->str+2);
+    Debug("%p HandleEvent \"%s\"\t", event->pd, event->str+2);
     size_t namelen;
     for( namelen=2 ; ; namelen++ ) {
     if( !event->str[namelen] ) { Debug("I\n");		return 0; }
