@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: mplayer.pl,v 1.18 2006/03/15 19:23:04 khlut Exp $
+# $Id: mplayer.pl,v 1.22 2006/03/26 10:32:46 khlut Exp $
 # Bind mplayer and mencoder with HTML object
 # ElphelOgm is used to receive and record rtp mjpeg video
 #
@@ -29,7 +29,8 @@ use POSIX ":sys_wait_h";
 ## Parameters from the config file ##
 $conffile="$ENV{HOME}/.mozilla/genres/mplayer.conf";
 %cfg=(
-  vo=>[x11, 'text', "Video output driver (x11,vx, see man mplayer)"],
+  vo=>[x11, 'text', "Video output driver (see mplayer -vo help)"],
+  ao=>["esd,arts,oss,alsa", 'text', "Audio output driver (see mplayer -ao help)"],
   not_ask=> [0, 'checkbox', "Never show file save dialog again."],
   out_file=>['%@-%Y-%m-%d_%H-%M-%S.avi', 'file', "Write video to file(s)",
      "%@  in the filename will be replaced to source URL\n"
@@ -48,7 +49,6 @@ require "config.pl";
 load_config();
 ## *** ##
 
-$v{timeout}=10;    #shut down mplayer after 10 seconds of silent
 $v{quittimeout}=2; #wait 2 seconds after send quit command or interruptable signal
 $v{play}=1;
 
@@ -61,6 +61,22 @@ my $thrid:shared;
 $^F=100; #to pass pipe to new processes
 pipe PIPE_OUT, PIPE_IN;
 
+sub localnet {
+    return $v{href}=~m!file://! || $v{href}=~m![a-zA-Z0-9]+://(127|192|10|172|232)\.! ;
+}
+sub gettimeout {
+    return localnet() ? 10 : 100;
+}
+sub progress {
+    use Gtk2 ;
+    return unless $v{xid} ;
+    my $W=Gtk2::Gdk::Window->foreign_new(hex $v{xid});
+    my (undef,undef,$w,$h) = $W->get_geometry;
+    my $gc=Gtk2::Gdk::GC->new($W);
+    $W->draw_rectangle ($gc, 1 , 0, 0, $w*$_[0]/10, 10);
+    Gtk2::Gdk->flush;
+}
+
 sub shut{
     lock $queue,$shutstate;
     !$shutstate++ && $v{save} && $shutstate++;
@@ -73,8 +89,8 @@ sub shut{
 	kill 2 => $sl2p if $sl2p;
     }else{			#if it again not respond kill it
         kill -9 => $slvp;
-	kill -9 => $sl2p if $sl2p;
-	print "\nrun=0\n";    
+	kill(-9 => $sl2p) || print "kill_error: $!\n" if $sl2p;
+#	print "\nrun=0\n";    
 	exit 0;	
     }
     return;
@@ -83,7 +99,7 @@ sub shut{
 sub normal_exit{
     $SIG{PIPE}='IGNORE';
     lock $queue,$shutstate;
-    alarm $v{$shutstate?'quittimeout':'timeout'};
+    alarm ($shutstate? $v{'quittimeout'} : gettimeout);
     close(PIPE_IN);
     close(PIPE_OUT);
     waitpid $slvp, 0;
@@ -97,7 +113,7 @@ sub normal_exit{
 	kill 9 => $sl2p;
 	waitpid $sl2p, 0;
     }
-    print "\nrun=0\n";
+#    print "\nrun=0\n";
     exit 0;
 }
 
@@ -159,13 +175,13 @@ sub ofilt_quit{ #child hangs
 sub ofilt{
     local $s;
     local $SIG{ALRM}=\&ofilt_quit;
-    alarm $v{timeout};
+    alarm gettimeout;
     $s="";
     print "run=1\n";
     do {
       while($_='', sysread( SLVR, $_, 1 ) || $!{EINTR}){
 	if( tr/\r/\n/ ){
-	    alarm($pausestate? 0 : $v{timeout});
+	    alarm($pausestate? 0 : gettimeout);
 	}
 	$s.=$_;
 	{lock $queue;
@@ -177,13 +193,19 @@ sub ofilt{
 	if($_ eq "\n") {
 	    if( $s =~ 'Failed' ) {
 	        print "error=$s";
-	    }elsif($s =~ m/^Playing \(.*\)\.$/){
+	    }elsif($s =~ m/^Playing (.*)\.$/){
 		$s=$1;
 		if($s !~ m!-|/dev!) {
 		    $s="file://".$s if $s !~ m|^[a-z0-9]*://|;
 		    print "url=$s";
 		}
+	    }elsif($s =~ m/^Cache fill:[[:space:]]*([0-9.]*)%/){
+		progress($1);
+		print "cachefill=".substr($s,11);
+	    }elsif($s =~ m/^Connecting to/){
+		print "status=$s";
 	    }elsif($s =~ m/^[AV]:/ || $s =~ m/^Pos:/){
+		$s =~ s/ \033\[J$//;
 		print "frame=$s";
 	    }else{
 		print $s;
@@ -288,15 +310,17 @@ while (<STDIN>){
 	    $OPTS.=" -lavdopts lowres='$v{lowres}'" if( $v{lowres} );
 	    $OPTS.=" -fps '$v{fps}'"		if( $v{fps} );
 	    $OPTS.=" -fixed-vo -loop ".(0+$v{loop}) if( exists $v{loop} );
+	    $OPTS.= localnet()? " -nocache" : " -cache 1000 -cache-min 10";
 	    if(( $cmd2=rtpcmd() )) {
 		$cmd2.=" -k $v{frameskip}" if $v{frameskip};
-	        $cmd2="|exec $cmd2 2>/dev/null ".fileno(PIPE_OUT)."<&- >/dev/fd/".fileno(PIPE_IN);
-		$cmd="mplayer $OPTS -vo $cfg{vo}[0] -zoom -nojoystick -slave -nomouseinput -cookies -nocache /dev/fd/".fileno(PIPE_OUT);
+	        $cmd2="|exec $cmd2 -i 2>/dev/null ".fileno(PIPE_OUT)."<&- >/dev/fd/".fileno(PIPE_IN);
+		$cmd="mplayer $OPTS -vo $cfg{vo}[0] -zoom -nojoystick -slave -nomouseinput -cookies /dev/fd/".fileno(PIPE_OUT);
 	        print "cmd=$cmd\n";
 	        print "cmd2=$cmd2\n";
 		runslv $cmd;		
 		$sl2p=open(SL2W,$cmd2);
 	    } else {
+		$OPTS.=" -ao $cfg{ao}[0]" if $cfg{ao}[0];
 		$cmd="mplayer $OPTS -vo $cfg{vo}[0] -zoom -nojoystick -slave -nomouseinput -cookies '$v{href}'";
 	        print "cmd=$cmd\n";
 	        runslv $cmd;
